------------------------------------------------------------------------
--     Nexysdemo.vhd -- Demonstrate basic Nexys function
------------------------------------------------------------------------
-- Author:  Clint Cole
--          Copyright 2006 Digilent, Inc.
------------------------------------------------------------------------
-- This module tests basic device function and connectivity on the Nexys
-- board. It was developed using the Xilinx WebPack tools.
--
--  Inputs:
--		mclk		- system clock (100Mhz Oscillator on Pegasus board)
--		bn			- buttons on the Pegasus board
--		swt		- switches on the Pegasus board (8 switches)
--
--  Outputs:
--		led		- discrete LEDs on the Pegasus board (8 leds)
--		an			- anode lines for the 7-seg displays on Pegasus
--		ssg		- cathodes (segment lines) for the displays on Pegasus
--
------------------------------------------------------------------------
-- Revision History:
--  01/15/2004(ClintC): created
------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Nexysdemo is
	Port (
		mclk    : in std_logic;
		btn     : in std_logic_vector(3 downto 0);
		swt     : in std_logic_vector(7 downto 0);
		led     : out std_logic_vector(7 downto 0);
		an      : out std_logic_vector(3 downto 0);
		ssg     : out std_logic_vector(7 downto 0);
		rxd		: in std_logic;
		txd		: out std_logic;
		
		sram_addr		: out 	std_logic_vector (22 downto 0);
		sram_data	: inout	std_logic_vector (15 downto 0);
		sram_cs 		: out		std_logic;
		sram_adv		: out		std_logic;
		sram_oe		: out		std_logic;
		sram_ub		: out		std_logic;
		sram_lb		: out		std_logic;
		sram_we		: out		std_logic;
		sram_clk		: out		std_logic;
		sram_wait	: in  	std_logic;
		sram_cre		: out		std_logic
	);
end Nexysdemo;

architecture Behavioral of Nexysdemo is

	------------------------------------------------------------------------
	-- Component Declarations
	------------------------------------------------------------------------

component T8052
	port(
		Clk			: in std_logic;
		Rst_n		: in std_logic;
		P0_in		: in  std_logic_vector(7 downto 0);
		P1_in		: in  std_logic_vector(7 downto 0);
		P2_in		: in  std_logic_vector(7 downto 0);
		P3_in		: in  std_logic_vector(7 downto 0);
		P0_out	: out std_logic_vector(7 downto 0);
		P1_out	: out std_logic_vector(7 downto 0);
		P2_out	: out std_logic_vector(7 downto 0);
		P3_out	: out std_logic_vector(7 downto 0);
		INT0		: in std_logic;
		INT1		: in std_logic;
		T0			: in std_logic;
		T1			: in std_logic;
		T2			: in std_logic;
		T2EX		: in std_logic;
		RXD			: in std_logic;
		RXD_IsO		: out std_logic;
		RXD_O		: out std_logic;
		TXD			: out std_logic;
		-- External XRAM Wishbone:
    XRAM_WE_O   : out std_logic;
    XRAM_STB_O  : out std_logic;
    XRAM_CYC_O  : out std_logic;
    XRAM_ACK_I  : in  std_logic;
    XRAM_DAT_O  : out std_logic_vector(7 downto 0);
    XRAM_ADR_O  : out std_logic_vector(15 downto 0);
    XRAM_DAT_I  : in  std_logic_vector(7 downto 0)
	);
end component;

component sram_access
		port (clk 			: in		std_logic;
					res 			: in 		std_logic;

					-- sram seite (fr 1 sram)

					 sri_adr	: out 	std_logic_vector (22 downto 0);
					 sri_data 	: inout std_logic_vector (15 downto 0);
					 sri_cs 		: out  	std_logic;
					 sri_adv 	: out  	std_logic;
					 sri_oe 		: out  	std_logic;
					 sri_ub 		: out  	std_logic;
					 sri_lb 		: out  	std_logic;
					 sri_we 		: out 	std_logic;
					 sri_clk 	: out  	std_logic;
					 sri_wait 	: in	 	std_logic;	

					-- fpga seite

--					ri_adr		: in  	std_logic_vector (22 downto 0);
--					ri_din		: in    std_logic_vector (15 downto 0);
--					ri_dout		: out 	std_logic_vector (15 downto 0);
--					ri_rw			: in		std_logic;
--					ri_trg		: in		std_logic;
--					ri_rdy		: out		std_logic;
					
					-- wishbone
					WE_I   : in std_logic;
					STB_I  : in std_logic;
					CYC_I  : in std_logic;
					ACK_O  : out  std_logic;
					DAT_I  : in std_logic_vector(15 downto 0);
					ADR_I  : in std_logic_vector(22 downto 0);
					DAT_O  : out std_logic_vector(15 downto 0)
				 );
end component;

	------------------------------------------------------------------------
	-- Signal Declarations
	------------------------------------------------------------------------

	signal clkdiv  : std_logic_vector(24 downto 0);
	signal cntr    : std_logic_vector(3 downto 0);
	signal cclk    : std_logic;
	signal dig     : std_logic_vector(6 downto 0);
	signal p3_in	: std_logic_vector(7 downto 0);
	signal rst		: std_logic;
	signal rst_n	: std_logic;
	
	-- T51
	signal T51_WE_O   : std_logic;
   signal T51_STB_O  : std_logic;
   signal T51_CYC_O  : std_logic;
   signal T51_ACK_I  : std_logic;
   signal T51_DAT_O  : std_logic_vector(7 downto 0);
   signal T51_ADR_O  : std_logic_vector(15 downto 0);
   signal T51_DAT_I  : std_logic_vector(7 downto 0);
	
	-- SRAM
	signal XRAM_WE_I   : std_logic;
   signal XRAM_STB_I  : std_logic;
   signal XRAM_CYC_I  : std_logic;
   signal XRAM_ACK_O  : std_logic;
   signal XRAM_DAT_I  : std_logic_vector(15 downto 0);
   signal XRAM_ADR_I  : std_logic_vector(22 downto 0);
   signal XRAM_DAT_O  : std_logic_vector(15 downto 0);
	
	------------------------------------------------------------------------
	-- Module Implementation
	------------------------------------------------------------------------

	begin

T51_inst: T8052
	port map(
		Clk		=> mclk,
		Rst_n		=> rst_n,
		P0_in		=> (others => '0'),
		P1_in		=> (others => '0'),
		P2_in		=> (others => '0'),
		P3_in		=> p3_in,
		P0_out	=> open,
		P1_out	=> open,
		P2_out	=> open,
		P3_out	=> open,
		INT0		=> '1',
		INT1		=> '1',
		T0			=> '1',
		T1			=> '1',
		T2			=> '1',
		T2EX		=> '1',
		RXD		=> rxd,
		RXD_IsO	=> open,
		RXD_O		=> open,
		TXD		=> txd,
		-- External XRAM Wishbone:
    XRAM_WE_O   => T51_WE_O,
    XRAM_STB_O  => T51_STB_O,
    XRAM_CYC_O  => T51_CYC_O,
    XRAM_ACK_I  => T51_ACK_I,
    XRAM_DAT_O  => T51_DAT_O,
    XRAM_ADR_O  => T51_ADR_O,
    XRAM_DAT_I  => T51_DAT_I
	);

	sram: sram_access
		port map(clk => mclk,
					res => rst,
					sri_adr => sram_addr,
					sri_data => sram_data,
					sri_cs => sram_cs,
					sri_adv => sram_adv,
					sri_oe => sram_oe,
					sri_ub => sram_ub,
					sri_lb => sram_lb,
					sri_we => sram_we,
					sri_clk => sram_clk,
					sri_wait => sram_wait,
					WE_I => XRAM_WE_I,
					STB_I => XRAM_STB_I,
					CYC_I => XRAM_CYC_I,
					ACK_O => XRAM_ACK_O,
					DAT_I => XRAM_DAT_I,
					ADR_I => XRAM_ADR_I,
					DAT_O => XRAM_DAT_O
				 );

	-- sram connections
	XRAM_WE_I <= T51_WE_O;
	XRAM_STB_I <= T51_STB_O;
	XRAM_CYC_I <= T51_CYC_O;
	T51_ACK_I <= XRAM_ACK_O;
	XRAM_DAT_I <= "00000000" & T51_DAT_O;
	XRAM_ADR_I <= "0000000" & T51_ADR_O;
	T51_DAT_I <= XRAM_DAT_O(7 downto 0);

	sram_cre		<= '0';

	-- MCS-51 basic baud rate detection needs RXD at P3.0
	p3_in <= (0 => rxd, others => '0');

	rst <= btn(0);
	rst_n <= not rst;

	-- Digilent code below

	led <= swt;
	
	dig <=    
         "0111111" when cntr = "0000" else
			"0000110" when cntr = "0001" else
			"1011011" when cntr = "0010" else
			"1001111" when cntr = "0011" else
			"1100110" when cntr = "0100" else
			"1101101" when cntr = "0101" else
			"1111101" when cntr = "0110" else
			"0000111" when cntr = "0111" else
			"1111111" when cntr = "1000" else
			"1101111" when cntr = "1001" else
			"0000000";

   ssg(6 downto 0) <= not dig;

	an <= btn;

	ssg(7) <= '0';

	-- Divide the master clock (100Mhz) down to a lower frequency.
	process (mclk)
		begin
			if mclk = '1' and mclk'Event then
				clkdiv <= clkdiv + 1;
			end if;
		end process;

	cclk <= clkdiv(24);

	process (cclk)
		begin
			if cclk = '1' and cclk'Event then
				if cntr = "1001" then
				    cntr <= "0000";
				else
				    cntr <= cntr + 1;
				end if;
			end if;
			
		end process;

end Behavioral;

