-- Simple (and slow) Wishbone interface to the Micron MT45W8MW16 PSDRAM. Tested with the T51 processor core.
-- Based on code from the Audio-DSP project and modified for wishbone bus by Andreas Schwarz (andreas@andreas-s.net)

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity sram_access is
	port (
		clk 		: in std_logic;
		res 		: in std_logic;

		-- SRAM
		sri_adr	: out std_logic_vector (22 downto 0);
		sri_data	: inout std_logic_vector (15 downto 0);
		sri_cs 	: out std_logic;
		sri_adv	: out std_logic;
		sri_oe	: out std_logic;
		sri_ub	: out std_logic;
		sri_lb	: out std_logic;
		sri_we	: out std_logic;
		sri_clk	: out std_logic;
		sri_wait	: in std_logic;

		-- Wishbone bus
		WE_I   : in std_logic;
		STB_I  : in std_logic;
		CYC_I  : in std_logic;
		ACK_O  : out std_logic;
		DAT_I  : in std_logic_vector(15 downto 0);
		ADR_I  : in std_logic_vector(22 downto 0);
		DAT_O  : out std_logic_vector(15 downto 0)
	);
end sram_access;

architecture RTL of sram_access is

	type ram_states is (
		idle, 
	
		write_00,
		write_01, 
		write_02, 
		write_03,
		write_04,
		write_05,
		
		read_00,
		read_01, 
		read_02,
		read_03,
		read_04,
		read_05,
		
		wait_for_stb_neg
	);

	signal ram_state	: ram_states;
	
	signal sram_gce	: std_logic;

	signal ri_rdy		: std_logic; -- unused

begin
	sri_ub 	<= sram_gce;
	sri_lb 	<= sram_gce;
	sri_cs  <= sram_gce;
	-- sri_adr <= ri_adr;
	sri_clk		<= '0';

	process (clk, res)
	begin
		if res = '1' then

			ram_state <= idle;			
			sri_data 	<= (others => 'Z');
			sri_adr 	<= (others => '0');
			sri_oe 		<= '1';
			sram_gce 	<= '1';
			sri_we 	 	<= '1';
			sri_adv		<= '1';

			DAT_O 	<= (others => '0');
			ri_rdy 		<= '0';
			
			ACK_O <= '0';

		elsif rising_edge(clk) then
		
			case ram_state is
				when idle 			=>	if STB_I = '1' then
													sri_adr <= ADR_I;
													sri_adv <= '0';
													
													if WE_I = '1' then
														ram_state	<= write_00;
														sri_data		<= (others => 'Z');
														sram_gce 	<= '0';
														sri_oe		<= '1';
														sri_we		<= '0';
														ri_rdy		<= '0';
													else
														ram_state	<= read_00;
														sri_data		<= (others => 'Z');
														sram_gce		<= '1';
														sri_oe		<= '1';
														sri_we		<= '1';
														ri_rdy		<= '0';
													end if;
															
												else
													sri_data	<= (others => 'Z');
													sram_gce	<= '1';
													sri_oe	<= '1';
													sri_we	<= '1';
													sri_adv	<= '1';
													ri_rdy	<= '1';
												end if;
												
												ACK_O <= '0'; -- reset ACK

				when write_00 	=>			sri_data	<= DAT_I;
												sram_gce	<= '0';
												sri_oe  	<= '1';
												sri_we  	<= '0';
												ri_rdy	<= '0';
												ram_state	<= write_01;

				when write_01 	=>  
												sram_gce	<= '0';
												sri_oe	<= '1';
												sri_we	<= '0';
												ri_rdy	<= '0';
												ram_state	<= write_02;
															

				when write_02 	=>
												sram_gce <= '0';
												sri_oe  	<= '1';
												sri_we  	<= '0';
												ri_rdy 	<= '0';
												ram_state	<= write_03;


				when write_03 	=>
												sram_gce	<= '0';
												sri_oe	<= '1';
												sri_we	<= '0';
												ri_rdy	<= '0';
												ram_state <= write_04;															

				when write_04 	=>
												sram_gce	<= '1';
												sri_oe  	<= '1';
												sri_we  	<= '0';
												ri_rdy 	<= '0';
												ram_state <= write_05;			

				when write_05 	=>  		sri_data	<= (others => 'Z');
												sram_gce	<= '1';
												sri_oe  	<= '1';
												sri_we  	<= '1';
												ri_rdy 	<= '1';
												ACK_O		<= '1';
												ram_state <= wait_for_stb_neg;

				when read_00	=>
												sri_data	<= (others => 'Z');
												sram_gce	<= '0';
												sri_oe	<= '1';
												sri_we	<= '1';
												ri_rdy	<= '0';
												ram_state <= read_01;

				when read_01 	=>
                              
												sram_gce	<= '0';
												sri_oe	<= '0';
												sri_we	<= '1';
												ri_rdy	<= '0';
												ram_state <= read_02;

				when read_02 	=>
												sram_gce	<= '0';
												sri_oe	<= '0';
												sri_we	<= '1';
												ri_rdy	<= '0';
												ram_state <= read_03;
															
				when read_03 	=>
												sram_gce	<= '0';
												sri_oe	<= '0';
												sri_we	<= '1';
												ri_rdy	<= '0';
												ram_state <= read_04;


				when read_04 	=>			DAT_O	<= sri_data;
												sram_gce	<= '0';
												sri_oe	<= '0';
												sri_we	<= '1';
												ri_rdy	<= '0';
												ram_state <= read_05;	
															

				when read_05 	=>
												sram_gce	<= '1';
												sri_oe	<= '1';
												sri_we	<= '1';
												ri_rdy	<= '1';
												ACK_O		<= '1';
												ram_state <= wait_for_stb_neg;	

				when wait_for_stb_neg =>
												-- wait for master to negate STB_O
												if STB_I = '0' then
													ACK_O <= '0';
													ram_state <= idle;		
												end if;

				when others 	=>			ram_state	<= idle;
												ri_rdy		<= '0';
			end case;

		end if;


	end process;

end RTL;
